function [Asub,Aenz,Info_m]=getExpandedA(S,Sreg,Vin_index,Vout_index,EnzID)

%Converting the stoichiometric coefficient matrix of overall reactions into
%the corresponding A matrix of elementary reactions.  The transporters corresponding 
%to the input and output must be last columns in S.  The influx is going to 
%be reversible, while the outflux(es)is (are) irreversible
%
%The Info_m is the housekeeping matrix providing the following information
%of each reaction:
%  The first column: index of the reaction
%  The second column: index of the free enzyme catalyzing the reaction
%  The third  and Fourth columns: start and end indices of the block Vuni's of the
%   elementary reactions
%  The fifth and sixth columns: start and end indices of the block enzyme
%   complexes involved in the reaction
%  The last column: category of the reaction:(1)intracellular rxn;
%   (2)Transporting in;(3)Transporting out;(4) regulation
%
%The transporters of pathway substrates are also have MM kinetics
%
%Incorporating the inhibition
%--------------------------------------
%Last revision Oct 31, 2007
%Linh Tran, UCLA
%Matthew Rizk, UCLA
%--------------------------------------

[nMetab,nNetRxns]=size(S);

if isempty(EnzID)  %no input for indices of enzymes catalyzing the rxns
    EnzID=[1:1:nNetRxns]';  %different enzyme catalyzes different rxn  
end

nin_=length(Vin_index);
nout_=length(Vout_index);
nInterRxns=nNetRxns-nin_-nout_;
nRevRxns=nInterRxns+nin_;
totalRevEnz=length(unique(EnzID(1:nRevRxns)));

vtmp=find(S>0);
Spro=zeros(nMetab,nNetRxns);
Spro(vtmp)=S(vtmp);

vtmp=find(S<0);
Ssub=zeros(nMetab,nNetRxns);
Ssub(vtmp)=S(vtmp);

vnsub=sum(Ssub);
vnpro=sum(Spro);
rcount=1;
ccount=totalRevEnz+1;
Aenz=[];
Asub=[];

for k=1:nInterRxns
    Info_m(k,[1:2 3 5 7])=[k,EnzID(k),rcount,ccount,1];  
    if vnsub(k)==-2 & vnpro(k)==2
    %two substrates and two products: 4 steps,4 enzyme complexes, and 8 Vuni 
        blockAsub=zeros(8,nMetab);
        subID=getExpandedIndices(Ssub(:,k));
        blockAsub(1,subID(1))=1;
        blockAsub(3,subID(2))=1;

        proID=getExpandedIndices(Spro(:,k)); 
        blockAsub(6,proID(1))=1;
        blockAsub(8,proID(2))=1;
                
        blockAenz=zeros(8,4);
        blockAenz([1 8],1)=1;
        blockAenz(2:3,2)=1; 
        blockAenz(4:5,3)=1;
        blockAenz(6:7,4)=1;
    else
        blockAenz=zeros(6,3);
        blockAenz([1 6],1)=1;
        blockAenz(2:3,2)=1; 
        blockAenz(4:5,3)=1;
       
        blockAsub=zeros(6,nMetab);
        subID=getExpandedIndices(Ssub(:,k));
        if vnsub(k)==-2 & vnpro(k)==1
        %2 substrates and 1 products: 3 steps,3 enzyme complexes, and 6 Vuni    
            blockAsub(1,subID(1))=1;
            blockAsub(3,subID(2))=1;
 
            proID=getExpandedIndices(Spro(:,k));
            blockAsub(6,proID)=1;
        else  %1 substrates        
            blockAsub(1,subID)=1;
            proID=getExpandedIndices(Spro(:,k));
            
            if vnpro(k)==1  %1 product
                blockAsub(6,proID)=1;
            else             %2 products
                blockAsub(4,proID(1))=1;
                blockAsub(6,proID(2))=1;                
            end
        end
    end
    if vnsub(k)==-2 & vnpro(k)==3
    %two substrates and three products: 5 steps,5 enzyme complexes, and 10 Vuni 
        blockAsub=zeros(10,nMetab);
        subID=getExpandedIndices(Ssub(:,k));
        blockAsub(1,subID(1))=1;
        blockAsub(3,subID(2))=1;
       
        proID=getExpandedIndices(Spro(:,k)); 
        blockAsub(6,proID(1))=1;
        blockAsub(8,proID(2))=1;
        blockAsub(10,proID(3))=1;
                
        blockAenz=zeros(10,5);
        blockAenz([1 10],1)=1;
        blockAenz(2:3,2)=1; 
        blockAenz(4:5,3)=1;
        blockAenz(6:7,4)=1;
        blockAenz(8:9,5)=1;
    end
        
    if vnsub(k)<-2 || vnpro(k)>3
        error('\nThe program limits to maximum of 2 reactants and 3 products per reaction\n');
    end
    
    Asub=[Asub;blockAsub];
    [nr,ne]=size(blockAenz);
    Aenz(rcount:rcount+nr-1,EnzID(k))=blockAenz(:,1);
    Aenz(rcount:rcount+nr-1,ccount:ccount+ne-2)=blockAenz(:,2:ne);
    rcount=rcount+nr;
    ccount=ccount+ne-1;
    Info_m(k,[4 6])=[rcount-1,ccount-1];
end

for k=1:nin_
    Info_m(k+nInterRxns,[1:2 3 5 7])=[k+nInterRxns,EnzID(k+nInterRxns),rcount,ccount,2]; 
    
    blockAsub=zeros(6,nMetab);
    subID=getExpandedIndices(Ssub(:,Vin_index(k)));
    blockAsub(1,subID)=1;
    proID=getExpandedIndices(Spro(:,Vin_index(k)));
    if vnpro(Vin_index(k))==1  %1 product
        blockAsub(6,proID)=1;
    else             %2 products
        blockAsub(4,proID(1))=1;
        blockAsub(6,proID(2))=1;                
    end
    
    %blockAenz=ones(2,1);
    blockAenz=zeros(6,3);
    blockAenz([1 6],1)=1;
    blockAenz(2:3,2)=1; 
    blockAenz(4:5,3)=1;
    
    Asub=[Asub;blockAsub];
    [nr,ne]=size(blockAenz);
    Aenz(rcount:rcount+nr-1,EnzID(Vin_index(k)))=blockAenz(:,1);
    Aenz(rcount:rcount+nr-1,ccount:ccount+ne-2)=blockAenz(:,2:ne);
    rcount=rcount+nr;
    ccount=ccount+ne-1;
    Info_m(k+nInterRxns,[4 6])=[rcount-1,ccount-1];    
end

Aenz_out=[zeros(rcount-1,nout_);eye(nout_)];  %corresponding for transporters of outfluxes
for k=1:length(Vout_index)
    Info_m(k+nRevRxns,[1:2 3:4 7])=[k+nRevRxns,EnzID(k+nRevRxns),rcount,rcount,3];
    
    subID=find(Ssub(:,Vout_index(k)));
    blockAsub=zeros(1,nMetab);  %assume both ref and target state have the same concentration of pathway substrate(s)
    blockAsub(1,subID)=1;   
    
    %blockAenz=zeros(1,ccount-1);  %only add row for Vuni, but no column for enzyme
    
    Asub=[Asub;blockAsub];
    Aenz=[Aenz;zeros(1,ccount-1)];
    rcount=rcount+1;  %only one Vuni=Vout      
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%FOR INHIBITION REGULATION
if ~isempty(Sreg)
    [xinh,yinh]=find(Sreg<0);
    vtmp=find(Sreg<0);

    for j=1:length(vtmp)
        nNormalEnz=ccount-1;
        if Sreg(vtmp(j))==-1
            %competitive inhibition:  1 step, 1 enzyme complex, and 2 Vuni
            Info_m(j+nNetRxns,[1:2 3 5 7])=[yinh(j),EnzID(yinh(j)),rcount,ccount,4];
            blockAinhsub=zeros(2,nMetab);
            blockAinhsub(1,xinh(j))=1;
            
            blockAinhenz1=zeros(2,nNormalEnz);
%             vlocated_=find(Info_m(1:nNetRxns,1)==yinh(j));
            blockAinhenz1(1,Info_m(yinh(j),2))=1;

            blockAinhenz2=zeros(2,1);
            blockAinhenz2(2,1)=1;            
         
        elseif Sreg(vtmp(j))==-2
            %uncompetitive inhibition:  1 step, 1 enzyme complex, and 2 Vuni
            Info_m(j+nNetRxns,[1:2 3 5 7])=[yinh(j),EnzID(yinh(j)),rcount,ccount,5];
            blockAinhsub=zeros(2,nMetab);
            blockAinhsub(1,xinh(j))=1;
            
            blockAinhenz1=zeros(2,nNormalEnz);
%             vlocated_=find(Info_m(1:nNetRxns,1)==yinh(j));
            blockAinhenz1(1,Info_m(yinh(j),5))=1;
            
            blockAinhenz2=zeros(2,1);
            blockAinhenz2(2,1)=1;            
         
        elseif Sreg(vtmp(j))==-3
            %mixed inhibition:  2 steps, 2 enzyme complexes, and 4 Vuni
            Info_m(j+nNetRxns,[1:2 3 5 7])=[yinh(j),EnzID(yinh(j)),rcount,ccount,6];
            blockAinhsub=zeros(4,nMetab);
            blockAinhsub([1 3],xinh(j))=1;
            
            blockAinhenz1=zeros(4,nNormalEnz);
            %vlocated_=find(Info_m(1:nNetRxns,1)==yinh(j));
            blockAinhenz1(1,Info_m(yinh(j),2))=1;
            blockAinhenz1(3,Info_m(yinh(j),5))=1;
            
            blockAinhenz2=zeros(4,2);
            blockAinhenz2(2,1)=1; 
            blockAinhenz2(4,2)=1;
        else
            error('\nThe program only accepts inhibition labeled with -1 or -2 or -3\n');
        end
        Asub=[Asub;blockAinhsub];
        Aenz=[Aenz;blockAinhenz1];
        [nr2,ne2]=size(blockAinhenz2);
        Aenz(rcount:rcount+nr2-1,ccount:ccount+ne2-1)=blockAinhenz2;
        Aenz_out=[Aenz_out;zeros(nr2,nout_)];
        rcount=rcount+nr2;
        ccount=ccount+ne2;
        Info_m(j+nNetRxns,[4 6])=[rcount-1,ccount-1];
    end
end

%stack the block corresponding to outflux transporters at the end
Aenz=[Aenz,Aenz_out];  
Info_m(nRevRxns+1:nRevRxns+nout_,5:6)=repmat([ccount:1:ccount+nout_-1],2,1)';        
           
%------------------------------------------------------
function v_ind=getExpandedIndices(v_input)

n=length(v_input);
v_input=abs(v_input);
v_ind=[];
for k=1:n
    if v_input(k)==1
        v_ind=[v_ind;k];
    elseif v_input(k)>1
        v_ind=[v_ind;repmat(k,v_input(k),1)];
    end
end
                
        
        
        
        
        
        
        
        
        
        
             
      
        
        
        
        
        
    